"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const alpha_command_bus_1 = require("alpha-command-bus");
const assert = require("assert");
exports.DEFAULT_RESULT_HANDLER = (result, context) => {
    context.response.send(result);
};
exports.DEFAULT_ERROR_HANDLER = (result, context) => {
    context.response.status(500).send(result);
};
class Mapper {
    constructor(commandBus, config = {}) {
        this.commandBus = commandBus;
        this.config = config;
        assert.ok(this.commandBus instanceof alpha_command_bus_1.CommandBus, 'command bus must be defined and be an instance of CommandBus');
    }
    map(mapper, config) {
        return (request, response) => __awaiter(this, void 0, void 0, function* () {
            const command = mapper(request);
            const context = {
                command,
                request,
                response
            };
            try {
                const result = yield this.commandBus.handle(command);
                const localResultHandler = (config && config.resultHandler);
                const parentResultHandler = (this.config && this.config.resultHandler) || exports.DEFAULT_RESULT_HANDLER;
                if (localResultHandler) {
                    yield localResultHandler(result, context, parentResultHandler);
                }
                else {
                    yield parentResultHandler(result, context);
                }
            }
            catch (e) {
                const localErrorHandler = (config && config.errorHandler);
                const parentErrorHandler = (this.config && this.config.errorHandler) || exports.DEFAULT_ERROR_HANDLER;
                if (localErrorHandler) {
                    localErrorHandler(e, context, parentErrorHandler);
                }
                else {
                    parentErrorHandler(e, context);
                }
            }
        });
    }
}
exports.Mapper = Mapper;
//# sourceMappingURL=Mapper.js.map